module Assembler where

import Instruction
import Table

assemble c =
    resolvAddress is table
    where
      result = assembleInstructions 0 (initialTable "undeclared label ") c
      is = fst result
      table = snd result

assembleInstructions address table ((ins, comment):cs) = 
    ((address, machineCodes, ins, comment) : fst next, snd next)
    where
      machineCodes = assembleInstruction ins
      machineCodesLength = length machineCodes
      newTable = registerLabel table ins address
      next = assembleInstructions (address + machineCodesLength) newTable cs
assembleInstructions address table [] = ([], table)

registerLabel table (Label l) address = updateTable table l ((), address)
registerLabel table _ address = table

resolvAddress ((address, machineCodes, ins, comment):is) table =
    case ins of
      (OperandString1 "JUMP" l) ->
          let
              labelAddress = (offsetTable table l)
              labelAddressCode = [div labelAddress 16, mod labelAddress 16]
          in
            (address, [0xf] ++ labelAddressCode, ins, comment)
      _ -> (address, machineCodes, ins, comment)
    : resolvAddress is table
resolvAddress [] table = []


assembleError = error . ("assemble error: " ++)

assembleInstruction (Operand0 "KA") = [0x0]
assembleInstruction (Operand0 "AO") = [0x1]
assembleInstruction (Operand0 "CH") = [0x2]
assembleInstruction (Operand0 "CY") = [0x3]
assembleInstruction (Operand0 "AM") = [0x4]
assembleInstruction (Operand0 "MA") = [0x5]
assembleInstruction (Operand0 "M+") = [0x6]
assembleInstruction (Operand0 "M-") = [0x7]
assembleInstruction (OperandInt1 "TIA" op1) = [0x8, op1]
assembleInstruction (OperandInt1 "AIA" op1) = [0x9, op1]
assembleInstruction (OperandInt1 "TIY" op1) = [0xa, op1]
assembleInstruction (OperandInt1 "AIY" op1) = [0xb, op1]
assembleInstruction (OperandInt1 "CIA" op1) = [0xc, op1]
assembleInstruction (OperandInt1 "CIY" op1) = [0xd, op1]
assembleInstruction (OperandString1 "JUMP" _) = [0xf, 0, 0]
assembleInstruction (OperandString1 "CAL" "RSTO") = [0xe, 0x0]
assembleInstruction (OperandString1 "CAL" "SETR") = [0xe, 0x1]
assembleInstruction (OperandString1 "CAL" "RSTR") = [0xe, 0x2]
assembleInstruction (OperandString1 "CAL" "INPT") = [0xe, 0x3]
assembleInstruction (OperandString1 "CAL" "CMPL") = [0xe, 0x4]
assembleInstruction (OperandString1 "CAL" "CHNG") = [0xe, 0x5]
assembleInstruction (OperandString1 "CAL" "SIFT") = [0xe, 0x6]
assembleInstruction (OperandString1 "CAL" "ENDS") = [0xe, 0x7]
assembleInstruction (OperandString1 "CAL" "ERRS") = [0xe, 0x8]
assembleInstruction (OperandString1 "CAL" "SHTS") = [0xe, 0x9]
assembleInstruction (OperandString1 "CAL" "LONS") = [0xe, 0xa]
assembleInstruction (OperandString1 "CAL" "SUND") = [0xe, 0xb]
assembleInstruction (OperandString1 "CAL" "TIMR") = [0xe, 0xc]
assembleInstruction (OperandString1 "CAL" "DSPR") = [0xe, 0xd]
assembleInstruction (OperandString1 "CAL" "DEM-") = [0xe, 0xe]
assembleInstruction (OperandString1 "CAL" "DEM+") = [0xe, 0xf]
assembleInstruction (Label _) = []
assembleInstruction x = assembleError ("unkown instruction " ++ show x)

