module AssemblyParser where

import Char

import Lexer
import Instruction

assemblyParse = assemblyParseInstruction . removeNewLine .  assemblyRemoveLineComment . assemblyFirstLex

assemblyRemoveLineComment = removeTokensBetween (\x -> any (x==) [(TokenSymbol "#"), (TokenSymbol ";")]) (==TokenNewLine) 0 

assemblyFirstLex [] = []
assemblyFirstLex s =
    if fst alphas /= "" then
        TokenIdentifier (map toUpper (fst alphas)) : assemblyFirstLex (snd alphas)
    else if fst newline /= "" then
             TokenNewLine : assemblyFirstLex (snd newline)
         else if fst symbol /= "" then
                  TokenSymbol (fst symbol) : assemblyFirstLex (snd symbol)
              else
                  assemblyFirstLex (tail s)
    where
      alwaysFalse x = False
      alphas = tokenerWord fp fp s ""
          where
            fp x = isAlphaNum x || any (x==) ['+', '-', '_']
      newline = tokenerWord isNewLine alwaysFalse s ""
          where
            isNewLine x = any (x==) "\r\n"
      symbol = tokenerWord fp alwaysFalse s ""
          where
            fp x = ((not.isAlphaNum) x) && (not.isSpace) x && not (any (x==) ['+', '-'])

assemblyParseInstruction ((TokenIdentifier ins):tokens)
    | any (ins==) ["KA", "AO", "CH", "CY", "AM", "MA", "M+", "M-"] =
        (Operand0 ins, ""): assemblyParseInstruction tokens

assemblyParseInstruction ((TokenIdentifier ins):(TokenIdentifier op):tokens)
    | any (ins==) ["TIA", "AIA", "TIY", "AIY", "CIA", "CIY"] =
        (OperandInt1 ins (readHexOperand op), ""): assemblyParseInstruction tokens

assemblyParseInstruction ((TokenIdentifier ins):(TokenIdentifier op):tokens)
    | any (ins==) ["JUMP", "CAL"] =
        (OperandString1 ins op, ""): assemblyParseInstruction tokens

assemblyParseInstruction ((TokenIdentifier label):(TokenSymbol ":"):tokens) =
    (Label label, ""): assemblyParseInstruction tokens

assemblyParseInstruction (s:tokens) = lexError ("unkown instruction " ++ show s)

assemblyParseInstruction [] = []

-- for char code free
readHexOperand "0" = 0
readHexOperand "1" = 1
readHexOperand "2" = 2
readHexOperand "3" = 3
readHexOperand "4" = 4
readHexOperand "5" = 5
readHexOperand "6" = 6
readHexOperand "7" = 7
readHexOperand "8" = 8
readHexOperand "9" = 9
readHexOperand "A" = 10
readHexOperand "B" = 11
readHexOperand "C" = 12
readHexOperand "D" = 13
readHexOperand "E" = 14
readHexOperand "F" = 15
readHexOperand s = lexError ("invalid operand " ++ s)
