module Lexer where

import Char

data Token =
    TokenIdentifier String |
    TokenSymbol String |
    TokenNumber Int |
    TokenNewLine
    deriving (Show, Eq)

lexer = removeNewLine . removeComment . firstLex

lexError = error . ("lexer error: " ++)

removeNewLine (TokenNewLine:tokens) = removeNewLine tokens
removeNewLine (token:tokens) = token : removeNewLine tokens
removeNewLine [] = []

removeComment = removeLineComment . removeBlockComment

removeLineComment = removeTokensBetween (==(TokenSymbol "//")) (==TokenNewLine) 0
removeBlockComment = removeTokensBetween (==(TokenSymbol "/*")) (==(TokenSymbol "*/")) 0

removeTokensBetween startTokenPredicate endTokenPredicate n (token:tokens)
    | startTokenPredicate token && n == 0 = removeTokensBetween startTokenPredicate endTokenPredicate (n + 1) tokens
removeTokensBetween startTokenPredicate endTokenPredicate n (token:tokens)
    | endTokenPredicate token && n > 0 = removeTokensBetween startTokenPredicate endTokenPredicate (n - 1) tokens
removeTokensBetween startTokenPredicate endTokenPredicate n (token:tokens)
    | n > 0 = removeTokensBetween startTokenPredicate endTokenPredicate n tokens
removeTokensBetween startTokenPredicate endTokenPredicate n (token:tokens) =
    token : removeTokensBetween startTokenPredicate endTokenPredicate n tokens
removeTokensBetween startTokenPredicate endTokenPredicate 0 [] = []
removeTokensBetween startTokenPredicate endTokenPredicate n [] = lexError "unterminated comment"

firstLex [] = []
firstLex s =
    if fst number /= "" then
        TokenNumber (read (fst number)) : firstLex (snd number)
    else if fst alphas /= "" then
             TokenIdentifier (fst alphas) : firstLex (snd alphas)
         else if fst newline /= "" then
                  TokenNewLine : firstLex (snd newline)
              else if fst symbols /= "" then
                       TokenSymbol (fst symbols) : firstLex (snd symbols)
                   else if fst symbol /= "" then
                            TokenSymbol (fst symbol) : firstLex (snd symbol)
                        else
                            firstLex (tail s)
    where
      alwaysFalse x = False
      alphas = tokenerWord isAlpha isAlphaNum s ""
      number = tokenerWord isDigit isAlphaNum s ""
      newline = tokenerWord isNewLine alwaysFalse s ""
          where
            isNewLine x = any (x==) "\r\n"
      symbol = tokenerWord fp alwaysFalse s ""
          where
            fp x = ((not.isAlphaNum) x) && ((not.isSpace) x)
      symbols = tokenerWord fp fp s ""
          where
            fp x = ((not.isAlphaNum) x) && ((not.isSpace) x) &&
                   not (any (x==) "{}()[];,")

tokenerWord fp sp (c:cs) "" | fp c = tokenerWord fp sp cs [c]
tokenerWord fp sp (c:cs) "" = ("", cs)
tokenerWord fp sp (c:cs) s | sp c = tokenerWord fp sp cs (s ++ [c])
tokenerWord fp sp cs s = (s, cs)
